import tw2.core as twc, testapi
import webob as wo
import os
from nose.tools import raises, eq_
from tw2.core.template import reset_engine_name_cache

# TBD: only test engines that are installed
engines = ['cheetah', 'kid', 'genshi', 'mako']

kid_prefix = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n"""
def strip_prefix(prefix, s):
    return s[len(prefix):] if isinstance(s, basestring) and s.startswith(prefix) else s

class TestWD(twc.Widget):
    test = twc.Param(default='bob')

class TestTemplate(object):
    def setUp(self):
        testapi.setup()

    def _check_render(self, template, data, expected, engine=None):
        if engine:
            mw = twc.make_middleware(None, preferred_rendering_engines=[engine])
            testapi.request(1, mw)
        out = twc.template.EngineManager().render(template, 'string', data)
        assert(isinstance(out, unicode))
        assert out == expected, out

    def test_auto_select_engine(self):
        #im not sure why this test fails with kid, but this is the error:
        #Traceback (most recent call last):
        #File "/.../lib/python2.5/site-packages/nose-0.11.0-py2.5.egg/nose/case.py", line 183, in runTest
        #self.test(*self.arg)
        #File "/.../tw2core-percious/tests/test_template.py", line 22, in _check_render
        #out = twc.template.EngineManager().render(template, 'string', data)
        #File "/.../src/tw2core-percious/tw2/core/template.py", line 44, in render
        #template = self[engine_name].load_template(template_path)
        #File "/.../lib/python2.5/site-packages/TurboKid-1.0.4-py2.5.egg/turbokid/kidsupport.py", line 151, in load_template
        #tclass = mod.Template
        #AttributeError: 'module' object has no attribute 'Template'

        engs = engines[:]
        engs.remove('kid')
        for engine in engs:
            #set up the default renderers
            yield self._check_render, 'tw2.core.test_templates.simple', {'test':engine}, '<p>TEST %s</p>'%engine, engine

    @raises(twc.template.EngineError)
    def test_auto_select_unavailable_engine(self):
        engine = 'mako'
        self._check_render('tw2.core.test_templates.simple_genshi', {'test':engine}, '<p>TEST %s</p>'%engine, engine)

    def test_auto_select_cache_works(self):
        engine='genshi'
        args = 'tw2.core.test_templates.simple_genshi', 'string', {'test':engine}
        em = twc.template.EngineManager()
        out = em.render(*args)
        assert(isinstance(out, unicode))
        assert out == '<p>TEST genshi</p>', out
        out = em.render(*args)
        assert(isinstance(out, unicode))
        assert out == '<p>TEST genshi</p>', out

    def test_auto_select_unavailable_engine_not_strict(self):
        engine = 'mako'
        mw = twc.make_middleware(None, preferred_rendering_engines=[engine], strict_engine_selection=False)
        testapi.request(501, mw)
        self._check_render('tw2.core.test_templates.simple_genshi', {'test':'blah!'}, '<p>TEST blah!</p>')

    def test_engines(self):
        for engine in engines:
            print "Testing %s..." % engine
            out = twc.template.EngineManager().render('%s:tw2.core.test_templates.simple_%s' % (engine, engine), 'string', {'test':'test1'})
            out = strip_prefix(kid_prefix, out)
            assert(isinstance(out, unicode))
            assert(out == '<p>TEST test1</p>')

    def test_engines_unicode(self):
        for engine in engines:
            print "Testing %s..." % engine
            out = twc.template.EngineManager().render('%s:tw2.core.test_templates.simple_%s' % (engine, engine), 'string', {'test':'test\u1234'})
            out = strip_prefix(kid_prefix, out)
            assert(out == '<p>TEST test\u1234</p>')

    def test_engine_dupe(self):
        em = twc.template.EngineManager()
        t = em[engines[0]]
        try:
            em.load_engine(engines[0])
            assert(False)
        except twc.EngineError, e:
            assert(str(e) == "Template engine '%s' is already loaded" % engines[0])

    def test_engine_notfound(self):
        try:
            t = twc.template.EngineManager()['fred']
            assert(False)
        except twc.EngineError, e:
            eq_(str(e), "No template engine for 'fred'")

    def test_extra_vars(self):
        eng = twc.template.EngineManager()
        for engine in engines[:3]: #mako is exempt
            print "Testing %s..." % engine
            eng.load_engine(engine, extra_vars_func=lambda: {'test':'wobble'})
            out = eng.render('%s:tw2.core.test_templates.simple_%s' % (engine, engine), 'string', {})
            out = strip_prefix(kid_prefix, out)
            assert(out == '<p>TEST wobble</p>')

    def test_nesting(self):
        "Check that templates can be correctly nested, in any combination"
        eng = twc.template.EngineManager()
        for outer in engines:
            for inner in engines:
                print 'Testing %s on %s' % (inner, outer)
                test = eng.render('%s:tw2.core.test_templates.simple_%s' % (inner, inner), outer, {'test':'test1'})
                test = strip_prefix(kid_prefix, test)
                out = eng.render('%s:tw2.core.test_templates.simple_%s' % (outer, outer), 'string', {'test':test})
                out = strip_prefix(kid_prefix, out)
                print out
                assert(out == '<p>TEST <p>TEST test1</p></p>')

    def test_widget_display(self):
        twc.core.request_local()['middleware'] = twc.make_middleware(None)
        mtest = TestWD(id='x')
        for eng in engines:
            test = mtest.req()
            test.template = '%s:tw2.core.test_templates.inner_%s' % (eng, eng)
            out = strip_prefix(kid_prefix, test.display())
            assert(out == '<p>TEST bob</p>')

    def test_widget_nesting(self):
        twc.core.request_local()['middleware'] = twc.make_middleware(None)
        for outer in engines:
            for inner in engines:
                test = twc.CompoundWidget(id='x',
                    template = '%s:tw2.core.test_templates.widget_%s' % (outer, outer),
                    children=[
                        TestWD(id='y', template='%s:tw2.core.test_templates.inner_%s' % (inner, inner)),
                    ]
                )
                assert(test.display().replace(kid_prefix, '') == '<p>TEST <p>TEST bob</p></p>')

    def test_genshi_abs(self):
        test_dir = os.path.sep.join(__file__.split(os.path.sep)[:-1])
        fname = os.path.sep.join([test_dir, 'test.html'])
        twc.Widget(template='genshi_abs:%s' % fname).display()
